package env;

import java.io.IOException;
import java.io.PrintWriter;

import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.objectweb.telosys.auth.TelosysAUTH;
import org.objectweb.telosys.common.Telosys;
import org.objectweb.telosys.common.TelosysServlet;
import org.objectweb.telosys.dal.TelosysDAL;
import org.objectweb.telosys.rpl.TelosysRPL;
import org.objectweb.telosys.uil.TelosysUIL;

/**
 * <p>
 * WebApp initialization
 * </p>
 * <p>
 * Description : <br>
 * Initialization servlet for the current WebApp <br>
 * init Telosys layers ( screens definitions, services, connections pool, ... )
 * etc ...
 * </p>
 */

public class InitServlet extends TelosysServlet {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	private static final String CONTENT_TYPE = "text/html";

	// -----------------------------------------------------------------------------
	public void init() throws ServletException {
		try {

			info("===========================================================================");
			info("=== InitServlet : init()");

			ServletConfig servletConfig = getServletConfig();

			if (servletConfig == null) {
				error("=== InitServlet : cannot get ServletConfig : getServletConfig() returns null ! ");
				return;
			}

			printContextInfo(servletConfig);

			// =========================================================================================
			info("===========================================================================");
			info("=== TELOSYS Version : " + Telosys.getVersion());
			info("===========================================================================");

			info("=== Main Factory Creation ...");

			// --- External Factory creation
			MainFactory extFactory = new MainFactory();

			info("=== Telosys Initialization ...");
			Telosys.init(extFactory, servletConfig);

			// --- Init the different layers
			info("=== Telosys authentication layer initialization ...");
			TelosysAUTH.init();

			info("=== Telosys U.I.L. initialization ...");
			TelosysUIL.init(getServletContext());

			info("=== Telosys R.P.L. initialization ...");
			TelosysRPL.init();

			info("=== Telosys D.A.L. initialization ...");
			TelosysDAL.init();

		} catch (Exception e) {
			error("Exception in initialization ! ");
			error("Exception = " + e.toString() + " - Message = "
					+ e.getMessage());
			e.printStackTrace();
		} catch (Throwable t) {
			error("Throwable exception in initialization ! ");
			error("Throwable exception = " + t.toString() + " - Message = "
					+ t.getMessage());
			t.printStackTrace();
		}

		info("===========================================================================");
		info(" End of Telosys initialization");
		info(" . Web App Context Name    = " + Telosys.getWebAppName());
		info(" . Web App Root Directory  = " + Telosys.getWebAppRootDir());
		info("===========================================================================");

		// =========================================================================================
		info("");
		info("===========================================================================");
		info("=== Application Data Initialization ...");
		info("===========================================================================");
		// try
		// {
		// AppData.init();
		// } catch (TelosysException ex)
		// {
		// error("Telosys Exception : " + ex.getMessage());
		// ex.printStackTrace();
		// }
		// DataSet ds = ApplicationDataHolder.getDataSet(AppData.AGENCIES_TER);
		// if (ds != null)
		// {
		// info(" . " + AppData.AGENCIES_TER + " : " + ds.getRowCount() + "
		// row(s) ");
		// }

		info("===========================================================================");
		info("=== End of Application Data Initialization");
		info("===========================================================================");
	}

	// -----------------------------------------------------------------------------
	// --- Destruction de la servlet => Nettoyage des ressources
	// -----------------------------------------------------------------------------
	public void destroy() {
		info("===========================================================================");
		info("=== TELOSYS : End of InitServlet ( destroy ) ");
		info("===========================================================================");
	}

	// -----------------------------------------------------------------------------
	// --- Requte HTTP GET
	// -----------------------------------------------------------------------------
	public void doGet(HttpServletRequest request, HttpServletResponse response)
			throws ServletException, IOException {
		trace("InitServlet.doGet()...");
		process(request, response);
	}

	// -----------------------------------------------------------------------------
	// --- Requte HTTP POST
	// -----------------------------------------------------------------------------
	public void doPost(HttpServletRequest request, HttpServletResponse response)
			throws ServletException, IOException {
		trace("InitServlet.doPost()...");
		process(request, response);
	}

	// -----------------------------------------------------------------------------
	// --- Rponse HTML pour une requte HTTP
	// -----------------------------------------------------------------------------
	// -----------------------------------------------------------------------------
	private void printContextInfo(ServletConfig config) {
		ServletContext context = config.getServletContext();
		if (context != null) {
			info("=== Server Info  : " + context.getServerInfo());
			info("=== Version      : " + context.getMajorVersion() + "."
					+ context.getMinorVersion());
			info("=== Context Name : " + context.getServletContextName());
			info("=== Real Path    : " + context.getRealPath(""));
		} else {
			info("=== Context not found ! ");
			error("Context not found");
		}
	}

	// -----------------------------------------------------------------------------
	private void printLine(PrintWriter out, String sLib, String sVal)
			throws ServletException, IOException {
		out.print("<tr><td>");
		out.print(sLib);
		out.print("</td><td>");
		out.print(sVal);
		out.println("</td></tr>");
	}

	// -----------------------------------------------------------------------------
	private void process(HttpServletRequest request,
			HttpServletResponse response) throws ServletException, IOException {
		PrintWriter out = response.getWriter();

		response.setContentType(CONTENT_TYPE);

		out.println("<html>");
		out.println("<h1>Telosys configuration : </h1>");

		out.println("<h3>Authentication layer : </h3>");
		out.println("<table>");
		printLine(out, "Realm name", TelosysAUTH.getRealmName());
		printLine(out, "First page", TelosysAUTH.getFirstPage());
		out.println("</table>");

		out.println("<hr>");
		out.println("</html>");
	}

}